//=======================================================================================
// ____          ____    __    ______     __________   __      __       __        __
// \    \       |    |  |  |  |   _   \  |___    ___| |  |    |  |     /  \      |  |
//  \    \      |    |  |  |  |  |_)   |     |  |     |  |    |  |    /    \     |  |
//   \    \     |    |  |  |  |   _   /      |  |     |  |    |  |   /  /\  \    |  |
//    \    \    |    |  |  |  |  | \  \      |  |     |   \__/   |  /  ____  \   |  |____
//     \    \   |    |  |__|  |__|  \__\     |__|      \________/  /__/    \__\  |_______|
//      \    \  |    |   ________________________________________________________________
//       \    \ |    |  |  ______________________________________________________________|
//        \    \|    |  |  |         __          __     __     __     ______      _______
//         \         |  |  |_____   |  |        |  |   |  |   |  |   |   _  \    /  _____)
//          \        |  |   _____|  |  |        |  |   |  |   |  |   |  | \  \   \_______
//           \       |  |  |        |  |_____   |   \_/   |   |  |   |  |_/  /    _____  |
//            \ _____|  |__|        |________|   \_______/    |__|   |______/    (_______/
//
//  This file is part of VirtualFluids. VirtualFluids is free software: you can
//  redistribute it and/or modify it under the terms of the GNU General Public
//  License as published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
//  VirtualFluids is distributed in the hope that it will be useful, but WITHOUT
//  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//
//  SPDX-License-Identifier: GPL-3.0-or-later
//  SPDX-FileCopyrightText: Copyright © VirtualFluids Project contributors, see AUTHORS.md in root folder
//
//! \addtogroup gpu_PostProcessor PostProcessor
//! \ingroup gpu_core core
//! \{
#ifndef FORCE_CALCULATIONS_H
#define FORCE_CALCULATIONS_H

#include "Parameter/Parameter.h"
#include "basics/utilities/UbSystem.h"

#include <iostream>
#include <stdio.h>

class CudaMemoryManager;

class ForceCalculations
{
public:
    ForceCalculations(Parameter* para);
    void calcPIDControllerForForce(Parameter* para, CudaMemoryManager* cudaMemoryManager);
    void printForcing(Parameter* para);

private:
    double vx1Targed; //!< target velocity.
    double Kpcrit; //Kp critical
    double Tcrit;  //the oscillation period 
    double Tn;
    double Tv;
    double e;
    double Ta;
    double Kp;
    double Ki;
    double Kd;
    double y;
    double esum;
    double eold;
    bool isPID;
};

#endif /* FORCE_CALCULATIONS_H */

//! \}
