//=======================================================================================
// ____          ____    __    ______     __________   __      __       __        __
// \    \       |    |  |  |  |   _   \  |___    ___| |  |    |  |     /  \      |  |
//  \    \      |    |  |  |  |  |_)   |     |  |     |  |    |  |    /    \     |  |
//   \    \     |    |  |  |  |   _   /      |  |     |  |    |  |   /  /\  \    |  |
//    \    \    |    |  |  |  |  | \  \      |  |     |   \__/   |  /  ____  \   |  |____
//     \    \   |    |  |__|  |__|  \__\     |__|      \________/  /__/    \__\  |_______|
//      \    \  |    |   ________________________________________________________________
//       \    \ |    |  |  ______________________________________________________________|
//        \    \|    |  |  |         __          __     __     __     ______      _______
//         \         |  |  |_____   |  |        |  |   |  |   |  |   |   _  \    /  _____)
//          \        |  |   _____|  |  |        |  |   |  |   |  |   |  | \  \   \_______
//           \       |  |  |        |  |_____   |   \_/   |   |  |   |  |_/  /    _____  |
//            \ _____|  |__|        |________|   \_______/    |__|   |______/    (_______/
//
//  This file is part of VirtualFluids. VirtualFluids is free software: you can
//  redistribute it and/or modify it under the terms of the GNU General Public
//  License as published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
//  VirtualFluids is distributed in the hope that it will be useful, but WITHOUT
//  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//
//  SPDX-License-Identifier: GPL-3.0-or-later
//  SPDX-FileCopyrightText: Copyright © VirtualFluids Project contributors, see AUTHORS.md in root folder
//
//! \addtogroup gpu_cuda_helper cuda_helper
//! \{
//! \author Soeren Peters
//=======================================================================================
#ifndef CUDA_CudaTimer_H
#define CUDA_CudaTimer_H

#include <cuda_runtime.h>
#include <helper_functions.h>
#include <helper_cuda.h>

namespace vf::cuda
{

class CudaTimer
{
public:
    void createSdkTimer();
    void startSdkTimer();
    void stopSdkTimer(float &timeSinceLastStop,double &totalTime);
    void deleteSdkTimer();

    void createEventTimer();
    void startEventTimer();
    void stopEventTimer(float &timeSinceLastStop,double &totalTime);
    void deleteEventTimer();

private:
    StopWatchInterface *sdkTimer;
    double ftimeS = {0.0};

    cudaEvent_t stop_t;
    cudaEvent_t start_t;
    double ftimeE {0.0};

};

}

#endif

//! \}
